@@ -1,16 +1,31 @@
-import requests
+""" This module contains a Conn class to connect and
+retrive data from Eulerian Technologies API
+"""
+
 import os
 import time
 import gzip
-from pprint import pprint
-import ijson
-import json
-import csv
-import inspect
-import urllib
-from datetime import datetime, timedelta
 import re
+import urllib
+import csv
+from datetime import datetime, timedelta
+import logging
+
 import pandas as pd
+import requests
+import ijson
+
+from .internal.os import _is_skippable_request
+from .internal.request import _request_to_json
+
+# logger
+logger = logging.getLogger(__name__)
+logger.setLevel(logging.DEBUG)
+
+# stream handler
+ch = logging.StreamHandler()
+formatter = logging.Formatter('%(asctime)s:%(name)s:%(funcName)s:%(levelname)s: %(message)s')
+ch.setFormatter(formatter)
 
 class Conn:
     """Setup the connexion to Eulerian Technologies API.
@@ -26,9 +41,9 @@ class Conn:
     api_key : str, obligatory
         Your Eulerian Technologies user account API key
 
-    print_log : bool, optional
-        If set to False, will not print log messages
-        Default: True
+    log_level : str, optional
+        Log level accepted values: ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
+        Default: INFO
 
     Returns
     -------
@@ -39,181 +54,54 @@ class Conn:
         gridpool_name : str,
         datacenter  : str,
         api_key : str,
-        print_log : bool = True
+        log_level : str = "INFO"
     ):
-        self.__print_log = print_log
+        if not isinstance(log_level, str):
+            raise TypeError("log_level should be a string type")
+
+        allowed_log_level = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
+        log_level = log_level.upper()
+        if log_level not in allowed_log_level:
+            raise ValueError(f"Accepted log_level values= {', '.join(allowed_log_level)}")
+
+        ch.setLevel(log_level)
+        logger.addHandler(ch)
+
+        if not isinstance(gridpool_name, str):
+            raise TypeError("gridpool_name should be a string type")
         self.__gridpool_name = gridpool_name
-        self.__http_headers = { "Authorization" : f"Bearer {api_key}" }
+
+        if not isinstance(datacenter, str):
+            raise TypeError("datacenter should be a string type")
         self.__base_url = f"https://{gridpool_name}.api.eulerian.{datacenter}"
         self.__api_v2 = f"{self.__base_url}/ea/v2"
-        self.__api_key = api_key
 
+        if not isinstance(api_key, str):
+            raise TypeError("api_key should be a string type")
+
+        self.__api_key = api_key
+        self.__http_headers = { "Authorization" : f"Bearer {api_key}" }
+
+        self._check_credentials()
+        logger.info("Credentials ok")
+
+    def _check_credentials(self) -> None:
+        """Check credentials validity"""
         overview_url = f"{self.__api_v2}/er/account/authtree.json"
 
-        # checking credentials
-        self.__request_to_json(
+        # raise an error if API error or fail to load as json
+        _request_to_json(
             request_type = "get",
             url=overview_url,
-            headers=self.__http_headers
+            headers=self.__http_headers,
+            logger=logger
         )
 
-    def __remove_file(
-        self,
-        path2file : str
-    ) -> None:
-        """ Remove the given path2file
-
-        Parameters
-        ----------
-        path2file : str, obligatory
-            The output full path to file
-
-        Returns
-        -------
-            A boolean
-        """
-        if not isinstance(path2file, str):
-            raise TypeError(f"path2file={path2file} should be a str type")
-
-        if os.path.isfile(path2file):
-            self.__log(f"Deleting path2file={path2file}")
-            os.remove(path2file)
-
-    def __is_skippable_request(
-        self,
-        output_path2file : str,
-        override_file : bool
-    ) -> bool :
-        """ Load data from local file is exist and override_file is True
-
-        Parameters
-        ----------
-        output_path2file : str, obligatory
-            The output full path to file
-
-        override_file : bool, obligatory
-            Your assigned datacenter (com for Europe, ca for Canada) in Eulerian Technologies platform
-
-        Returns
-        -------
-            A boolean
-        """
-
-        if os.path.isfile(output_path2file):
-            if override_file:
-                self.__log(f'Local path2file={output_path2file} will be overridden with new data')
-                return 0
-
-            self.__log(f'Output_path2file={output_path2file} already exists, skipping download')
-            return 1
-
-        self.__log(f'path2file={output_path2file} not found, downloading new data')
-        return 0
-
-    def __log(
-        self,
-        log : str
-    ) -> None :
-        """ A simple logging mechanism
-
-        Parameters
-        ----------
-        log : str, obligatory
-            The log message to display
-
-        Returns
-        -------
-            Nothing, print a log if self.__print_log is True
-        """
-
-        if self.__print_log:
-            stack = inspect.stack()
-            frame = stack[1]
-            caller_func = frame.function
-            caller_mod = inspect.getmodule(frame[0])
-            log_msg = f"{time.ctime()}:{caller_mod.__name__}:{caller_func}: {log}"
-            print(log_msg)
-
-    @staticmethod
-    def __request_to_json(
-        request_type : str,
-        url : str,
-        headers : dict = {},
-        params : dict = {},
-        json_data : dict = {}
-    ) -> dict :
-        """ Make HTTP request and check for error
-
-        Parameters
-        ----------
-        request_type : str, obligatory
-            The type of request : get/post supported at the moment
-
-        headers : dict, optional
-            The dict to use as the request header
-
-        params : dict, optional
-            The dict to use as the request params (requests.get)
-
-        json_data : dict, optional
-            The dict to use as the request json params (requests.post)
-
-        Returns
-        -------
-            Request response loaded as JSON
-        """
-        request_map = {
-            'get' : requests.get,
-            'post' : requests.post
-        }
-        allowed_requests_type = ["get", "post"]
-
-        if request_type not in allowed_requests_type:
-            raise ValueError(f"request_type is not in {', '.join(allowed_requests_type)}")
-
-        if request_type == "get":
-            r = request_map["get"](
-                url=url,
-                headers=headers,
-                params=params
-            )
-
-        elif request_type == "post":
-            r = request_map["post"](
-                url=url,
-                headers=headers,
-                json=json_data
-            )
-
-        # if request cannot be converted into JSON
-        try:
-            r_json = r.json()
-
-        except Exception as e:
-            print(r.text)
-            raise(e)
-
-        else:
-            # potential errors from Eulerian Technologies API
-            if "error" in r_json.keys() and r_json["error"] \
-            or "success" in r_json.keys() and not r_json["success"] \
-            or "status" in r_json.keys() and r_json['status'].lower() == "failed":
-
-                http_status_code = r.status_code
-                print(f"Error[{http_status_code}] from Eulerian Technologies API")
-                pprint(r_json)
-                params = urllib.parse.urlencode(params) if params else ''
-                raise SystemError(f"Error for url={url}?{params}\
-                    json_data={pprint(json_data)}"
-                )
-
-        return r_json
-
     def download_realtime_report(
-            self,
-            website_name: str,
-            report_name: list,
-            payload : dict,
+        self,
+        website_name: str,
+        report_name: list,
+        payload : dict,
     ):
 
         """ Fetch realtime report data into a pandas dataframe
@@ -235,14 +123,14 @@ class Conn:
         """
 
         if not isinstance(website_name, str):
-            raise TypeError("website_name should be a string")
+            raise TypeError("website_name should be a str type")
 
         if not isinstance(report_name, str):
-            raise TypeError("report_name should be a string")
+            raise TypeError("report_name should be a str typoe")
 
 
         if not isinstance(payload, dict):
-            raise TypeError("payload should be a dict")
+            raise TypeError("payload should be a dict typoe")
 
         if not payload:
             raise ValueError("payload should not be empty")
@@ -251,11 +139,12 @@ class Conn:
         payload['ea-switch-datetorow'] = 1 # include the date in each row
         payload['ea-enable-datefmt'] = "%s" # format the date as an epoch timestamp
 
-        report_json = self.__request_to_json(
+        report_json = _request_to_json(
             request_type="get",
             url=report_url,
             params=payload,
             headers=self.__http_headers,
+            logger=logger
         )
 
         fields = [ field['name'] for field in report_json['data']['fields'] ]
@@ -272,15 +161,14 @@ class Conn:
         return df
 
     def download_datamining(
-            self,
-            website_name: str,
-            datamining_type: str,
-            jobrun_id = None,
-            payload = {},
-            status_waiting_seconds = 5,
-            output_directory = None,
-            override_file = False,
-            n_days_slice = 31,
+        self,
+        website_name: str,
+        datamining_type: str,
+        payload = None,
+        status_waiting_seconds = 5,
+        output_directory = None,
+        override_file = False,
+        n_days_slice = 31,
     ):
 
         """ Fetch datamining data from the API into a gzip compressed CSV file
@@ -303,7 +191,8 @@ class Conn:
             The local targeted  directory
 
         override_file : bool, optional
-            If set to True, will override output_path2file (if exists) with the new datamining content
+            If set to True, will override output_path2file (if exists)
+                with the new datamining content
             Default: False
 
         n_days_slice: int, optional
@@ -324,7 +213,6 @@ class Conn:
             date_format : str,
             dt_date_from: timedelta(),
             dt_date_to: timedelta(),
-            view_id: int,
             status_waiting_seconds: int,
         ) -> str :
             """ download locally JSON response from API and convert it into a gzipped CSV file """
@@ -333,12 +221,22 @@ class Conn:
             date_from_file = date_from.replace("/", "_")
             date_to_file = date_to.replace("/", "_")
 
-            output_filename = f"{website_name}_{datamining_type}_view_{view_id}_from_{date_from_file}_to_{date_to_file}.csv.gz"
+            output_filename = "_".join([
+                website_name,
+                datamining_type,
+                "view",
+                payload["view-id"],
+                "from",
+                date_from_file,
+                "to",
+                date_to_file,
+            ])+".csv.gz"
             output_path2file = os.path.join(output_directory, output_filename)
 
-            if self.__is_skippable_request(
+            if _is_skippable_request(
                 output_path2file=output_path2file,
-                override_file=override_file
+                override_file=override_file,
+                logger=logger
             ):
                 return output_path2file
 
@@ -347,13 +245,14 @@ class Conn:
 
             search_url = f"{self.__api_v2}/ea/{website_name}/report/{datamining_type}/search.json"
             search_url_debug = f"{self.__api_v2}/{self.__api_key}/ea/{website_name}/report/{datamining_type}/search.json"
-            self.__log(f"http get url {search_url_debug}?{urllib.parse.urlencode(payload, safe='/')}")
+            logger.info(f"search url {search_url_debug}?{urllib.parse.urlencode(payload, safe='/')}")
 
-            search_json = self.__request_to_json(
+            search_json = _request_to_json(
                 request_type="get",
                 url=search_url,
                 params=payload,
-                headers=self.__http_headers
+                headers=self.__http_headers,
+                logger=logger
             )
 
             jobrun_id = search_json["jobrun_id"]
@@ -366,43 +265,34 @@ class Conn:
                 status_waiting_seconds = 5
 
             while not ready:
-                self.__log(f'Waiting for jobrun_id={jobrun_id} to complete')
+                logger.info(f'Waiting for jobrun_id={jobrun_id} to complete')
                 time.sleep(status_waiting_seconds)
-                status_json = self.__request_to_json(
+                status_json = _request_to_json(
                     request_type="get",
                     url=status_url,
                     params=status_payload,
-                    headers=self.__http_headers
+                    headers=self.__http_headers,
+                    logger=logger
                 )
 
                 if status_json["jobrun_status"] == "COMPLETED":
                     ready = True
 
-            self.__log('Downloading data')
+            logger.info('Downloading data')
             download_url = f"{self.__api_v2}/ea/{website_name}/report/{datamining_type}/download.json"
             download_payload = { 'output-as-csv' : 0, 'jobrun-id' : jobrun_id }
-            output_path2file_temp = f"{output_path2file}.tmp"
 
-            with  open(
-                file=output_path2file_temp,
-                mode='wb'
-            ) as f:
-                r = requests.get(
-                        download_url,
-                        params = download_payload,
-                        headers = self.__http_headers,
-                        stream=True
-                )
+            download_url_debug = f"{self.__api_v2}/{self.__api_key}/ea/{website_name}/report/{datamining_type}/download.json"
+            logger.info(f"download url {download_url_debug}?{urllib.parse.urlencode(download_payload, safe='/')}")
 
-                if "Content-Length" in r.headers:
-                    self.__log(f"Content-Length is {int(r.headers['Content-Length'])/(1024*1024):.2f} MBs")
 
-                # stream the response to avoid out of memory error
-                for chunk in r.iter_content(1024 * 1024 * 5): # 5MB
-                    f.write(chunk)
+            req = urllib.request.Request(
+                    url=f"{download_url}?{urllib.parse.urlencode(download_payload)}",
+                    headers = self.__http_headers,
+            )
 
-            self.__log(f"JSON data downloaded into path2file={output_path2file_temp}")
-            self.__log("Converting JSON to CSV...")
+            if req.has_header('Content-Length'):
+                logger.info(f"Content-Length={int(req.get_header('Content-Length'))/(1024*1024):.2f} MBs")
 
             with gzip.open(
                 filename=output_path2file,
@@ -414,71 +304,61 @@ class Conn:
                     delimiter=';'
                 )
 
-                # stream JSON file and convert into CSV
-                with open(
-                    file=output_path2file_temp,
-                    mode="rb"
-                ) as f:
-
+                with urllib.request.urlopen(req) as f:
                     columns = []
-                    headers_object = ijson.items(f, "data.fields")
-                    for headers in headers_object:
-                        # working on header.name rather than header.header for consitency
-                        # because the latest is language specific
-                        for header in headers:
-                            # this allows to recover the name of the product param name
-                            # instead of the id
-                            if header["name"].startswith('productparam_'):
-                                match = re.search(
-                                    pattern = r'\s:\s([\w\W]+?)\s#\s(\d)+$',
-                                    string = header["header"]
-                                )
-                                prdp_name = match.group(1)
-                                prd_idx = int(match.group(2))-1 # start at 0
-                                header["name"] = f"productparam_{prdp_name}_{prd_idx}"
-                            # this allows to recover the name of the cgi param name
-                            # instead of the id
-                            if header["name"].startswith('cgiparam_'):
-                                match = re.search(
-                                    pattern = r'^[\w\W]+?\s:\s(.*)$',
-                                    string = header["header"],
-                                )
-                                cgip_name = match.group(1)
-                                header["name"] = f"cgiparam_{cgip_name}"
-                            # this allows to recover the name of the CRM param name
-                            # instead of the id
-                            if header["name"].startswith('iduserparam_'):
-                                match = re.search(
-                                    pattern = r'^[\w\W]+?\s:\s(.*)$',
-                                    string = header["header"],
-                                )
-                                iduserparam_name = match.group(1)
-                                header["name"] = f"iduserparam_{iduserparam_name}"
-                            # this allows to recover the name of the audience name
-                            # instead of the id
-                            if header["name"].startswith('cluster_'):
-                                match = re.search(
-                                    pattern = r'^[\w\W]+?\s:\s(.*)$',
-                                    string = header["header"],
-                                )
-                                cluster_name = match.group(1)
-                                header["name"] = f"cluster_{cluster_name}"
+                    objects = ijson.items(f, "data.fields.item") #.item is for ijson
+                    headers = (header for header in objects)
+                    # working on header.name rather than header.header for consitency
+                    # because the latest is language specific
+                    for header in headers:
+                        # this allows to recover the name of the product param name
+                        # instead of the id
+                        if header["name"].startswith('productparam_'):
+                            match = re.search(
+                                pattern = r'\s:\s([\w\W]+?)\s#\s(\d+)$',
+                                string = header["header"]
+                            )
+                            prdp_name = match.group(1)
+                            prd_idx = int(match.group(2))-1 # start at 0
+                            header["name"] = f"productparam_{prdp_name}_{prd_idx}"
+                        # this allows to recover the name of the cgi param name
+                        # instead of the id
+                        if header["name"].startswith('cgiparam_'):
+                            match = re.search(
+                                pattern = r'^[\w\W]+?\s:\s(.*)$',
+                                string = header["header"],
+                            )
+                            cgip_name = match.group(1)
+                            header["name"] = f"cgiparam_{cgip_name}"
+                        # this allows to recover the name of the CRM param name
+                        # instead of the id
+                        if header["name"].startswith('iduserparam_'):
+                            match = re.search(
+                                pattern = r'^[\w\W]+?\s:\s(.*)$',
+                                string = header["header"],
+                            )
+                            iduserparam_name = match.group(1)
+                            header["name"] = f"iduserparam_{iduserparam_name}"
+                        # this allows to recover the name of the audience name
+                        # instead of the id
+                        if header["name"].startswith('cluster_'):
+                            match = re.search(
+                                pattern = r'^[\w\W]+?\s:\s(.*)$',
+                                string = header["header"],
+                            )
+                            cluster_name = match.group(1)
+                            header["name"] = f"cluster_{cluster_name}"
 
-                            columns.append(header["name"])
-                        csvwriter.writerow(columns)
+                        columns.append(header["name"])
+                    csvwriter.writerow(columns)
 
-                with open(
-                    file=output_path2file_temp,
-                    mode="rb"
-                ) as f:
-                    rows_object = ijson.items(f, "data.rows")
-                    for rows in rows_object:
-                        for row in rows:
-                            csvwriter.writerow(row)
+                with urllib.request.urlopen(req) as f:
+                    objects = ijson.items(f, "data.rows.item") # .item is for ijson
+                    rows = (row for row in objects)
+                    for row in rows:
+                        csvwriter.writerow(row)
 
-            # removing temp JSON file
-            self.__remove_file(path2file=output_path2file_temp)
-            self.__log(f"Output csv path2file={output_path2file}")
+            logger.info(f"Output csv path2file={output_path2file}")
             return output_path2file
             # end of inner function
 
@@ -486,12 +366,22 @@ class Conn:
         l_allowed_datamining_types = ["order", "estimate", "isenginerequest", "actionlog", "scart"]
         date_format = "%m/%d/%Y"
 
+        if not isinstance(website_name, str):
+            raise TypeError("website_name should be a str type")
+
         if datamining_type not in l_allowed_datamining_types:
             raise ValueError(f"datamining_type={datamining_type} not allowed.\n\
                             Use one of the following: {', '.join(l_allowed_datamining_types)}")
 
         if not isinstance(n_days_slice, int) or n_days_slice < 0 :
-            raise TypeError(f"n_days_slice should be a positive integer")
+            raise TypeError("n_days_slice should be a positive integer")
+
+        if not isinstance(payload, dict) or not payload:
+            raise TypeError("payload should be a non-empty dict")
+
+        if output_directory:
+            if not isinstance(output_directory, str):
+                raise TypeError("output_directory should be a str type")
 
         date_from = payload["date-from"] if "date-from" in payload else None
         if not date_from:
@@ -501,6 +391,7 @@ class Conn:
         if not date_to:
             raise ValueError("missing parameter=date-from in payload object")
 
+
         n_days_slice = timedelta(days=n_days_slice)
         one_day_slice = timedelta(days=1)
 
@@ -517,7 +408,17 @@ class Conn:
             os.mkdir(output_directory)
 
         # marketing attribution rule id, default to 0
-        view_id = int(payload["view-id"]) if "view-id" in payload else 0
+        if "view-id" in payload:
+            payload["view-id"] = str(payload["view-id"])
+            match = re.match(
+                pattern=r'^[0-9]$',
+                string=payload["view-id"]
+            )
+            if not match:
+                raise ValueError("view-id should match ^[0-9]$")
+
+        else:
+            payload["view-id"] = "0"
 
         # To avoid overloading the API with huge requests
         # We split the requests into smallers timerantes "n_days_slice"
@@ -533,7 +434,6 @@ class Conn:
                 dt_date_from = dt_date_from,
                 dt_date_to = dt_tmp_date_to,
                 status_waiting_seconds = status_waiting_seconds,
-                view_id = view_id,
                 date_format = date_format,
             )
             l_path2file.append(output_path2file)
@@ -548,13 +448,13 @@ class Conn:
         return l_path2file
 
     def download_edw(
-            self,
-            query : str,
-            status_waiting_seconds=5,
-            ip : str = None,
-            output_path2file = None,
-            override_file = False,
-            jobrun_id = None,
+        self,
+        query : str,
+        status_waiting_seconds=5,
+        ip : str = None,
+        output_path2file = None,
+        override_file = False,
+        jobrun_id = None,
     ) -> str :
         """ Fetch edw data from the API into a gzip compressed file
 
@@ -572,10 +472,12 @@ class Conn:
 
         output_path2file: str, optional
             path2file where the data will be stored
-            If not set, the file will be created in the current working directory with a default name
+            If not set, the file will be created in the current
+                working directory with a default name
 
         override_file : bool, optional
-            If set to True, will override output_path2file (if exists) with the new datamining content
+            If set to True, will override output_path2file (if exists)
+                with the new datamining content
             Default: False
 
         jobrun_id : str, optional
@@ -586,7 +488,6 @@ class Conn:
         str
             The output_path2file containing the downloaded datamining data
         """
-
         if not isinstance(query, str):
             raise TypeError("query should be a string")
 
@@ -609,7 +510,11 @@ class Conn:
             )
         )
 
-        if not output_path2file:
+        if output_path2file:
+            if not isinstance(output_path2file, str):
+                raise TypeError("output_path2file should be a str type")
+
+        else:
             output_path2file = '_'.join([
                 "dw",
                 self.__gridpool_name,
@@ -617,17 +522,19 @@ class Conn:
                 "_".join(readers),
             ])+".cvs.gzip"
 
-            if self.__is_skippable_request(
+            if _is_skippable_request(
                 output_path2file=output_path2file,
-                override_file=override_file
+                override_file=override_file,
+                logger=logger
             ):
                 return output_path2file
 
-        # tmp file to store JSON response before transforming into CSV
-        output_path2file_temp = f"{output_path2file}.tmp"
+        if ip:
+            if not isinstance(ip, str):
+                raise ValueError("ip should be a str type")
 
-        if not ip:
-            self.__log(f"No ip provided\
+        else:
+            logger.warning("No ip provided\
                 \n Fetching external ip from https://api.ipify.org\
                 \nif using a vpn, please provide the vpn ip\
             ")
@@ -636,11 +543,12 @@ class Conn:
         edw_token_url = f"{self.__api_v2}/er/account/get_dw_session_token.json"
         payload = { 'ip' : ip }
 
-        edw_token_json = self.__request_to_json(
+        edw_token_json = _request_to_json(
             request_type="get",
             url=edw_token_url,
             headers=self.__http_headers,
-            params=payload
+            params=payload,
+            logger=logger
         )
 
         edw_token = edw_token_json["data"]["rows"][0][0]
@@ -652,18 +560,18 @@ class Conn:
         # no jobrun_id provided, we send the query to get one
         if not jobrun_id:
             search_url =  f"{self.__base_url}:981/edw/jobs"
-            self.__log(search_url)
 
             edw_json_params = {
                 "kind" : "edw#request",
                 "query" : query
             }
 
-            search_json = self.__request_to_json(
+            search_json = _request_to_json(
                 request_type="post",
                 url=search_url,
                 json_data=edw_json_params,
                 headers=edw_http_headers,
+                logger=logger
             )
 
             jobrun_id = search_json['data'][0]
@@ -676,86 +584,70 @@ class Conn:
         ready = False
 
         while not ready:
-            self.__log(f"Waiting for jobrun_id={jobrun_id} to complete")
+            logger.info(f"Waiting for jobrun_id={jobrun_id} to complete")
             time.sleep(status_waiting_seconds)
-            status_json = self.__request_to_json(
+            status_json = _request_to_json(
                 request_type="get",
                 url=status_url,
-                headers=edw_http_headers
+                headers=edw_http_headers,
+                logger=logger
             )
 
             if status_json["status"] == "Done":
                 ready = True
                 download_url = status_json["data"][1]
 
-        with open(
-            file=output_path2file_temp,
-            mode="wb"
-        ) as f:
-            r = requests.get(
-                    download_url,
-                    headers = self.__http_headers,
-                    stream=True
-            )
-            if 'Content-Length' in r.headers:
-                self.__log(f"Content-Length is {int(r.headers['Content-Length'])/(1024*1024):.2f} MBs")
 
-            for chunk in r.iter_content(1024*1024*5): # 5MB
-                f.write(chunk)
+        req = urllib.request.Request(
+            url=download_url,
+            headers = self.__http_headers,
+        )
 
-        self.__log(f"JSON data downloaded into path2file={output_path2file_temp}")
-        self.__log("Converting JSON to CSV...")
+
+        if req.has_header('Content-Length'):
+            logger.info(f"Content-Length={int(req.get_header('Content-Length'))/(1024*1024):.2f} MBs")
 
         # Stream tmp json file and build csv file
         with gzip.open(
             filename=output_path2file,
             mode="wt"
         ) as csvfile:
+
             csvwriter = csv.writer(
                 csvfile,
                 delimiter=';'
             )
 
-            # Get headers
-            with open(
-                file=output_path2file_temp,
-                mode="rb"
-            ) as f:
-                headers_object = ijson.items(f, "headers.schema")
-                for headers in headers_object:
-                    csv_headers = [header[1] for header in headers]
-                    csvwriter.writerow(csv_headers)
+            columns = []
+            with urllib.request.urlopen(req) as f:
+                objects = ijson.items(f, "headers.schema.item")
+                headers = (header for header in objects)
+                for header in headers:
+                    columns.append(header[1])
+                csvwriter.writerow(columns)
 
-            # Get rows
-            with open(
-                file=output_path2file_temp,
-                mode="rb"
-            ) as f:
-                rows_object = ijson.items(f, "rows")
-                for rows in rows_object:
-                    for row in rows:
-                        csvwriter.writerow(row)
+            with urllib.request.urlopen(req) as f:
+                objects = ijson.items(f, "rows.item")
+                rows = (row for row in objects)
+                for row in rows:
+                    csvwriter.writerow(row)
 
-            # Try to kill job, non blocking error
-            try:
-                with open(
-                    file=output_path2file_temp,
-                    mode="rb"
-                ) as f:
-                    uuid_object = ijson.items(f, "headers.uuid")
-                    for uuid in uuid_object:
-                        cancel_url = f"{search_url}/{uuid}/cancel"
-                        requests.post(
-                            url=cancel_url,
-                            headers=edw_http_headers
-                        )
+        # Try to kill job, non blocking error
+        with urllib.request.urlopen(req) as f:
+            objects = ijson.items(f, "headers.uuid.item")
+            uuids = (uuid for uuid in objects)
+            for uuid in uuids:
+                cancel_url = f"{search_url}/{uuid}/cancel"
+                try:
+                    requests.post(
+                        url=cancel_url,
+                        headers=edw_http_headers
+                    )
+                except Exception as e:
+                    logger.warning(f"Could not kill the process uuid={uuid}\n\
+                                    exception:{e}")
 
-            except Exception as e:
-                print(e)
-                print(f"Could not kill the process uuid={uuid}")
-
-        self.__remove_file(path2file=output_path2file_temp)
-        self.__log(f"Output csv path2file={output_path2file}")
+        logger.info(f"Output csv path2file={output_path2file}")
 
         return output_path2file
 
@@ -775,12 +667,15 @@ class Conn:
         dict
             A dict as { "view_id" : "view_name", ...}
         """
+        if not isinstance(website_name, str):
+            raise TypeError("website_name should be a string")
 
         view_url = f"{self.__api_v2}/ea/{website_name}/db/view/get_all_name.json"
-        view_json = self.__request_to_json(
+        view_json = _request_to_json(
             request_type="get",
             url=view_url,
-            headers=self.__http_headers
+            headers=self.__http_headers,
+            logger=logger
         )
 
         view_id_idx = view_json["data"]["fields"].index({"name" : "view_id"})
@@ -808,15 +703,18 @@ class Conn:
         dict
             A dict as { "website_prop" : "website_prop_value" }
         """
+        if not isinstance(website_name, str):
+            raise TypeError("website_name should be a string")
 
         website_url = f"{self.__api_v2}/ea/{website_name}/db/website/get_me.json"
-        website_json =  self.__request_to_json(
+        website_json =  _request_to_json(
             request_type="get",
             url=website_url,
-            headers=self.__http_headers
+            headers=self.__http_headers,
+            logger=logger
         )
 
         return  {
              website_json["data"]["fields"][i]["name"] : website_json["data"]["rows"][0][i]
                 for i in range(len(website_json["data"]["fields"]))
-        }
\ No newline at end of file
+        }
